unit UnitLanuage;

{
    Purpose:
    automatically traverse the control tree and extract/load
    the text for the controls.

    NOTES:
    this only includes support for a few control types.

}

interface

uses Forms, VCL.Controls, inifiles;

type TLanguage = class
    private
        class var ini : TCustomINIFile;
        class function GetCaption(c : TControl) : string;
        class procedure SetCaption(c : TControl; s : string);
        class function Escape(s : string) : string;
        class function UnEscape(s : string) : string;
    public
        class procedure LoadtoForm(f : TForm);
        class procedure ExtractFromForm(f : TForm);
        class function LanuageFileExists : boolean;
        class procedure DeleteLanguageFile;
end;

implementation

uses UnitMisc, SysUtils, VCL.StdCtrls, Vcl.ExtCtrls, VCL.Buttons;


const INI_NAME = 'language.ini';
class function TLanguage.LanuageFileExists : boolean;
begin
    result := SysUtils.FileExists(UnitMisc.GetAppPath + ini_name);
end;


class procedure TLanguage.DeleteLanguageFile;
var i : integer;
    newname : string;
begin
    i := 0;
    if TLanguage.LanuageFileExists then begin
        repeat
            newname := UnitMisc.GetAppPath + ini_name + '.' + Format('%.*d',[3, i]);
            if not FileExists(newname) then begin
                RenameFile(UnitMisc.GetAppPath + INI_NAME, newname);
                EXIT;
            end;
            inc(i);
        until false;
    end;
end;


class function TLanguage.Escape(s : string) : string;
var i, j : integer;
const SPECIAL = '='#13#10'\';
    CH_EQ = 1;
    CH_CR = 2;
    CH_LF = 3;
    CH_SLASH = 4;
begin
    result := '';
    for i := 1 to length(s) do begin
        j := pos(s[i], SPECIAL);
        case j of
        CH_EQ: result := result + '\' + s[i];
        CH_CR: result := result + '\r';
        CH_LF: result := result + '\n';
        CH_SLASH: result := result + '\\';
        else
            result := result + s[i];
        end;
    end;
end;

class function TLanguage.UnEscape(s : string) : string;
var i : integer;
    c : char;
    doEscapeNext : boolean;
begin
    result := '';
    for i := 1 to length(s) do begin
        c := s[i];
        if doEscapeNext then begin
            case c of
            'r': result := result + #13;
            'n': result := result + #10;
            else
                result := result + c;
            end;

            doEscapeNext := false;
        end else begin
            case c of
            '\': doEscapeNext := true;
            else
                result := result + c;
            end;
        end;

    end;
end;


class function TLanguage.GetCaption(c : TControl) : string;
begin
    result := '';
    if c is TEdit then begin
        result := TEdit(c).text;
    end else if c is TLabel then begin
        result := TLabel(c).Caption;
    end else if c is TPanel then begin
        result := TPanel(c).caption;
    end else if c is TButton then begin
        result := TButton(c).Caption;
    end else if c is TspeedButton then begin
        result := TSpeedButton(c).Caption;
    end else if c is TForm then begin
        result := TForm(c).Caption;
    end else if c is TGroupBox then begin
        result := TGroupBox(c).Caption;
    end else if c is TCheckbox then begin
        result := TCheckBox(c).Caption;
    end else if c is TRadioButton then begin
        result := TRadioButton(c).Caption;
    end;

    result := Escape(result);
end;

class procedure TLanguage.SetCaption(c : TControl; s : string);
begin
    s := TLanguage.UnEscape(s);

    if c is TEdit then begin
        TEdit(c).text := s;
    end else if c is TLabel then begin
        TLabel(c).Caption := s;
    end else if c is TPanel then begin
        TPanel(c).caption := s;
    end else if c is TButton then begin
        TButton(c).Caption := s;
    end else if c is TspeedButton then begin
        TSpeedButton(c).Caption := s;
    end else if c is TForm then begin
        TForm(c).Caption := s;
    end else if c is TGroupBox then begin
        TGroupBox(c).Caption := s;
    end else if c is TCheckbox then begin
        TCheckBox(c).Caption := s;
    end else if c is TRadioButton then begin
        TRadioButton(c).Caption := s;
    end;

end;

class procedure TLanguage.ExtractFromForm(f : TForm);
    procedure DoIt(psuedoName : string; wc : TWinControl);
    var i : integer;
        C : TControl;

        function HandleControl(c : TControl) : string;
        var s : string;
        begin
            s := TLanguage.GetCaption(c);

            result := psuedoName + c.Name +'-';
            if (s <> '') and (s <> c.Name) then begin
                ini.WriteString(result, 'Caption', s);
            end;
        end;

    var sName : string;
    begin
        HandleControl(wc);
        psuedoName := psuedoName + wc.Name + '-';

        for i := 0 to wc.ControlCount - 1 do begin
            c := wc.Controls[i];
            sName := HandleControl(c);

            if (c is TWinControl) then begin
                if (TWinControl(c).ControlCount > 0) and (c.Name <> '') then begin
                    DoIt(psuedoName, TWinControl(c));
                end;
            end;
        end;
    end;
begin
    ini := TMemIniFile.Create(UnitMisc.GetAppPath + INI_NAME, TEncoding.Unicode);

    DoIt('', F);

    ini.UpdateFile;
    UnitMisc.MyFree(ini);
end;

class procedure TLanguage.LoadtoForm(f : TForm);
    procedure DoIt(psuedoName : string; wc : TWinControl);
    var i : integer;
        C : TControl;

        function HandleControl(c : TControl) : string;
        var s : string;
        begin
            result := psuedoName + c.Name + '-';
            s := ini.ReadString(result, 'Caption', '');
            if (s <> '') then begin // don't save controls with default captions (same as control name)
                TLanguage.SetCaption(c, s);
            end;
        end;

    var sname : string;
    begin
        HandleControl(wc);
        psuedoName := psuedoName + wc.Name + '-';
        for i := 0 to wc.ControlCount - 1 do begin
            c := wc.Controls[i];
            sname := HandleControl(c);
            if (c is TWinControl) then begin
                if (TWinControl(c).ControlCount > 0) and (c.Name <> '') then begin
                    DoIt(sName, TWinControl(c));
                end;
            end;
        end;
    end;
begin
    ini := TMemIniFile.Create(UnitMisc.GetAppPath + INI_NAME, TEncoding.Unicode);

    DoIt('', F);

    UnitMisc.MyFree(ini)
end;

end.
